SystemOrganization addCategory: #FixUnderscores!

!MethodReference methodsFor: '*FixUnderscores' stamp: 'nil'!
fixLFInvisible
	"Replace invisible with space. Answer true if fixed or no fix necessary, false if manual fix required"

	| src ts |
	"Check if we do need to do anything"
	src := self actualClass sourceCodeAt: methodSymbol.
	(src includes: Character lf) ifFalse: [^true].

	"Chicken out if there is a literal underscore"
	"cm := self actualClass compiledMethodAt: methodSymbol.
	(cm hasLiteralSuchThat: [:lit | 
		lit = Character lf or: [lit isString and: [lit includes: Character lf]]]) ifTrue: [^false]."

	"Otherwise, replace underscores with :="
	src := src copyReplaceAll: Character lf asString with: Character space asString, Character cr asString.
	ts := self timeStamp.
	ts = '' ifTrue: [ts := nil].
	self actualClass 
		compile: src
		classified: ClassOrganizer default
		withStamp: ts 
		notifying: nil.

	^true
! !

!MethodReference methodsFor: '*FixUnderscores' stamp: 'nil'!
fixUnderscores
	"Replace underscores with :=. Answer true if fixed or no fix necessary, false if manual fix required"

	| src cm ts |
	"Check if we do need to do anything"
	src := self actualClass sourceCodeAt: methodSymbol.
	(src includes: $_) ifFalse: [^true].

	"Chicken out if there is a literal underscore"
	cm := self actualClass compiledMethodAt: methodSymbol.
	(cm hasLiteralSuchThat: [:lit | 
		lit = $_ or: [lit isString and: [lit includes: $_]]]) ifTrue: [^false].

	"Otherwise, replace underscores with :="
	src := src copyReplaceAll: '_' with: ':='.
	ts := self timeStamp.
	ts = '' ifTrue: [ts := nil].
	self actualClass 
		compile: src
		classified: ClassOrganizer default
		withStamp: ts 
		notifying: nil.

	^true
! !

Object subclass: #FixUnderscores
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FixUnderscores'!

!FixUnderscores class methodsFor: 'fixing' stamp: 'nil'!
arrowChar
	"FIXME: this should rather be (Character leftArrow)"
	^Character value: 16r8F! !

!FixUnderscores class methodsFor: 'tests' stamp: 'nil'!
asSeconds
	"Answer the seconds since the Squeak epoch: 1 January 1901"

	^ 12 asSeconds! !

!FixUnderscores class methodsFor: 'fixing' stamp: 'nil'!
fixFont: aFont
	| glyph underline |
	glyph := aFont characterFormAt: $_.
	"save arrow glyph to arrowChar codepoint"
	((glyph copy: (0@aFont ascent corner: glyph extent)) isAllWhite
		and: [(aFont characterFormAt: self arrowChar) isAllWhite])
			ifTrue: [aFont characterFormAt: self arrowChar put: glyph].
	"make underscore glyph"
	glyph fillWhite.
	underline := aFont ascent + 1.
	glyph fillBlack: (1@underline extent: glyph width-1@1).
	aFont characterFormAt: $_ put: glyph.! !

!FixUnderscores class methodsFor: 'fixing' stamp: 'nil'!
fixFonts
	"self fixFonts"

	StrikeFont allInstances
		do: [:fnt | self fixFont: fnt]
		displayingProgress: 'Fixing Bitmap Fonts'.! !

!FixUnderscores class methodsFor: 'fixing' stamp: 'nil'!
fixLFPackages: packageNames
	"FixUnderscores fixLFPackages: #('FixUnderscores')"

	| failed |
	failed := OrderedCollection new.

	packageNames
		do: [:pkgName | (PackageInfo named: pkgName) methods
			do: [:mRef | mRef fixLFInvisible ifFalse: [failed add: mRef]] 
			displayingProgress: pkgName]
		displayingProgress: 'Fixing ...'.

	failed isEmpty ifFalse: [
		MessageSet openMessageList: failed
			name: 'These methods with lf were not fixed'
			autoSelect: Character lf asString].! !

!FixUnderscores class methodsFor: 'fixing' stamp: 'nil'!
fixPackage: aPackageName
	^self fixPackages: {aPackageName}

! !

!FixUnderscores class methodsFor: 'fixing' stamp: 'nil'!
fixPackages: packageNames
	"FixUnderscores fixPackages: #('FixUnderscores' 'Bert')"

	| failed |
	failed := OrderedCollection new.

	packageNames
		do: [:pkgName | (PackageInfo named: pkgName) methods
			do: [:mRef | mRef fixUnderscores ifFalse: [failed add: mRef]] 
			displayingProgress: pkgName]
		displayingProgress: 'Fixing ...'.

	failed isEmpty ifFalse: [
		MessageSet openMessageList: failed
			name: 'These methods with literal underscores were not fixed'
			autoSelect: '_'].! !

!FixUnderscores class methodsFor: 'initialization' stamp: 'nil'!
initialize
	"self initialize"

	self fixFonts.
	self inform: 'Fonts were _fixed_.\The arrow glyph is now Character value ' withCRs,
		self arrowChar asInteger hex, ' ($', self arrowChar asString, ')'.
! !
FixUnderscores initialize!
